@@ -22,15 +22,15 @@
 #'    data (i.e., 432 blocks). During the randomization, because blocks 
 #'    maintain their relative locations in shuffleUnits (e.g., Days), block 1 
 #'    in the random set will be determined by sampling from a distribution of 
-#'    blocks 1,145,and 289, which each representing the first block of a given 
+#'    blocks 1,145,and 289, which each represent the first block of a given 
 #'    shuffleUnit (e.g., Day 1, Day 2, Day 3). All blocks in the randomized set
 #'    are decided in this fashion (e.g., block 2 of the randomized set is 
 #'    identified by sampling from a distribution of blocks 2, 146, and 290). 
 #'    Therefore, blocklength-units may never exceed 1 shuffleUnit 
 #'    (e.g., 25-hour blocks cannot be shuffled using shuffleUnit == "Days," 
 #'    but 1:24-hour blocks work just fine). Points within blocks are not 
-#'    redistributed. This particular shuffle.type (i.e., 2) is based off of 
-#'    the methodology described by Spiegel et al. 2016.
+#'    redistributed. Shuffle.types 1 & 2 are both variants of the randomization
+#'    methodology described by Spiegel et al. 2016.
 #' 
 #' Note that, if shuffle.type == 2, all dateTime values in individuals movement
 #'    paths described in x must be equidistant (e.g., relocations for 
@@ -61,19 +61,16 @@
 #'    coordinates must be arranged in the format of those in 
 #'    referencePointToPolygon output. Defaults to NULL.
 #' @param parallel Logical. If TRUE, sub-functions within the randomizePaths 
-#'    wrapper will be parallelized. Note that this can significantly speed up 
-#'    processing of relatively small data sets, but may cause R to crash due 
-#'    to lack of available memory when attempting to process large datasets. 
-#'    Defaults to FALSE.
+#'    wrapper will be parallelized. Defaults to FALSE.
 #' @param nCores Integer. Describes the number of cores to be dedicated to 
-#'    parallel processes. Defaults to the maximum number of cores available
-#'    (i.e., parallel::detectCores()).
+#'    parallel processes. Defaults to half of the maximum number of cores 
+#'    available (i.e., (parallel::detectCores()/2)).
 #' @param dataType Character string refering to the type of real-time-location 
 #'    data presented in x, taking values of "Point" or "Polygon." If 
 #'    dataType == "Point," individuals' locations are drawn from point.x and 
 #'    point.y. If argument == "Polygon," individuals' locations are drawn from 
 #'    poly.xy. Defaults to "Point."
-#' @param numVertices Numerical. If dataType == "Polygon," users must specify 
+#' @param numVertices Integer. If dataType == "Polygon," users must specify 
 #'    the number of vertices contained in each polygon. Defaults to 4. Note: 
 #'    all polygons must contain the same number of vertices.
 #' @param blocking Logical. If TRUE, prior to randomization, timepoints will be
@@ -82,12 +79,12 @@
 #'    randomization methodology will follow shuffle.type. If FALSE, paths will 
 #'    be randomized by sampling from observed timepoints. No timepoints will be
 #'    represented more than once in the randomized set. Defaults to TRUE.
-#' @param blockUnit Numerical. Describes the number blockUnits within each 
+#' @param blockUnit Integer. Describes the number blockUnits within each 
 #'    temporal block. Defaults to 1.
 #' @param blockLength Character string taking the values, "secs," "mins," 
 #'    "hours," "days," or "weeks." Describes the temporal unit associated with 
 #'    each block. Defaults to "hours."
-#' @param shuffle.type Numerical. Describes which shuffle.type is used to 
+#' @param shuffle.type Integer. Describes which shuffle.type is used to 
 #'    randomize the rand.input data set(s), given that blocking == TRUE 
 #'    (Note: this value is irrelevant if blocking == FALSE). Takes the values 
 #'    "0," "1," or "2," and defaults to 0. Descriptions of each shuffle.type 
@@ -105,10 +102,14 @@
 #'    location switching between ids (e.g., randomized xy locations for 
 #'    individual 1 will be generated by sampling from the entire dataset's 
 #'    location distribution). Defaults to TRUE.
-#' @param numRandomizations Numerical. The number of replicate data frames 
+#' @param numRandomizations Integer. The number of replicate data frames 
 #'    produced in output. Defaults to 1.
+#' @param reduceOutput Logical. If TRUE, to reduce output size, only "id," 
+#'    "x.rand," "y.rand," "dateTime," and "rand.rep" columns will be included 
+#'    in function output. Defaults to FALSE.
 #' @keywords data-processing contact
-#' @return Output is \code{x} appended with columns described below.
+#' @return If reduceOutput == FALSE, output is \code{x} appended with columns 
+#'    described below.
 #'    
 #'    \item{x.rand}{Randomized values taken from the \code{point.x} 
 #'    argument.}
@@ -118,8 +119,8 @@
 #'    argument.}
 #'    \item{rand.rep}{Randomization replicate.}
 #'    
-#'    If blocking == TRUE, the following codes are appended to the output
-#'    data frame described above:
+#'    If blocking == TRUE AND reduceOutput == FALSE, the following codes are 
+#'    appended to the output data frame described above:
 #'    
 #'    \item{block}{Integer ID describing unique blocks of time during which 
 #'    contacts occur.}
@@ -132,6 +133,9 @@
 #'    \item{blockLength}{Character string describing the length of blocks 
 #'    described by \code{blockLength} and \code{blockUnit} arguments.}
 #'    
+#'    If reduceOutput == TRUE, only \code{id}, \code{x.rand}, \code{y.rand},
+#'    \code{dateTime}, and \code{rand.rep} will be included in output.
+#'    
 #' @references Spiegel, O., Leu, S.T., Sih, A., and C.M. Bull. 2016. Socially 
 #'    interacting or indifferent neighbors? Randomization of movement paths to 
 #'    tease apart social preference and spatial constraints. Methods in 
@@ -160,22 +164,8 @@
 #'    indivPaths = TRUE, numRandomizations = 1) 
 #'    
 
-randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, point.y = NULL, poly.xy = NULL, parallel = FALSE, nCores = parallel::detectCores(), dataType = "Point", numVertices = 4, blocking = TRUE, blockUnit = "hours", blockLength = 1, shuffle.type = 0, shuffleUnit = "days", indivPaths = TRUE, numRandomizations = 1){
+randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, point.y = NULL, poly.xy = NULL, parallel = FALSE, nCores = (parallel::detectCores()/2), dataType = "Point", numVertices = 4, blocking = TRUE, blockUnit = "hours", blockLength = 1, shuffle.type = 0, shuffleUnit = "days", indivPaths = TRUE, numRandomizations = 1, reduceOutput = FALSE){
   
-  datetime.append1 = function(x){
-    timevec = x$dateTime
-    daySecondList = lubridate::hour(timevec) * 3600 + lubridate::minute(timevec) * 60 + lubridate::second(timevec) #This calculates a day-second
-    lub.dates = lubridate::date(x$dateTime)
-    dateseq = unique(lub.dates)
-    dayIDList = NULL
-    dayIDseq = seq(1,(length(dateseq)),1)
-    for(b in dayIDseq){
-      ID = rep(b,length(which(lub.dates == dateseq[which(dayIDseq == b)])))
-      dayIDList = c(dayIDList, ID)
-    } 
-    x$totalSecond = ((dayIDList - min(dayIDList))*86400) + daySecondList #This calculates the total second (the cumulative second across the span of the study's timeframe)
-    return(x)
-  }
   randomization<-function(x,y){
     
     dataType = unlist(unname(x[2]))
@@ -185,7 +175,9 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     blocking = unlist(unname(x[6]))
     shuffleUnit = unlist(unname(x[7]))
     blockUnit = unlist(unname(x[8]))
-    blockLength = as.integer(unlist(unname(x[9]))) #1/31 had to ass the as.integer call to avoid a "Error in (24)/blockLength : non-numeric argument to binary operator" error in the apply function.
+    blockLength = as.integer(unlist(unname(x[9]))) #1/31 had to add the as.integer call to avoid a "Error in (24)/blockLength : non-numeric argument to binary operator" error in the apply function.
+    reduceOutput = unlist(unname(x[10]))
+    
     y <- y[order(y$id, y$dateTime),]
     
     if(indivPaths == TRUE){
@@ -194,12 +186,12 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
       idVecSeq <- 1
     }
     numids <- length(idVecSeq)
-    idTab<- data.frame(idVecSeq)
+    idTab<- data.frame(idVecSeq, stringsAsFactors = TRUE)
     randomness<- apply(idTab,1,datasub.func, y, numids, blocking, shuffle.type, dataType, shuffleUnit, blockUnit, blockLength) #prior to 01/31, this was missing the "blockUnit" argument, resulting in an error. Obviously, this is now fixed.
     if(is.data.frame(randomness) == TRUE){
       randomxyCoords<-randomness
     }else{
-      randomxyCoords<-data.frame(data.table::rbindlist(randomness))
+      randomxyCoords<-data.frame(data.table::rbindlist(randomness), stringsAsFactors = TRUE)
     }
     
     if(blocking == TRUE & shuffle.type == 2){ #To account for potential length differences in blocks if shuffle.type ==2, the outputTab is essentially compiled earlier on in the function (in the datasub.func subfunction) 
@@ -209,6 +201,13 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     }
     outputTab$shuffle.type = shuffle.type #added 1/30/2019, to ensure that people remember what shuffle.type they set.
     outputTab$rand.rep = unlist(unname(x[1]))
+    
+    if(reduceOutput == TRUE){
+      outputTab <- data.frame(id = outputTab$id, x.rand = outputTab$x.rand, y.rand = outputTab$y.rand, dateTime = outputTab$dateTime, rand.rep = outputTab$rand.rep, stringsAsFactors = TRUE)
+    }else{
+      rownames(outputTab)<- seq(1, nrow(outputTab), 1) #ensure rownames are sequential. If output was reduced, this happened automatically.
+    }
+    
     return(outputTab)
   }
   datasub.func<-function(x,y,z, blocking, shuffle.type, dataType, shuffleUnit, blockUnit, blockLength){
@@ -220,16 +219,16 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(blocking == TRUE){
       if(shuffle.type == 0){ #This is the same as shuffle == FALSE. Blocks maintain their order in the dataset, but the xy locations within each block are randomized.
         blockSeq <-unique(idFrame$block)
-        blockTab <-data.frame(blockSeq)
+        blockTab <-data.frame(blockSeq, stringsAsFactors = TRUE)
         rand <-apply(blockTab,1,xy.assignNoShuff, idFrame, dataType, numVertices)
-        randomxy <- data.frame(data.table::rbindlist(rand))
+        randomxy <- data.frame(data.table::rbindlist(rand), stringsAsFactors = TRUE)
       }
       if(shuffle.type == 1){ #This rearranges the order of blocks in the dataset. xycoordinates within each block remain unchanged.
         blockSeq <-unique(idFrame$block)
         rand.block <- sample(blockSeq,length(blockSeq), replace = FALSE)
-        rand.blockTab <- data.frame(rand.block)
+        rand.blockTab <- data.frame(rand.block, stringsAsFactors = TRUE)
         rand <-apply(rand.blockTab,1,xy.assignShuff, idFrame, dataType, numVertices)
-        randomxy <- data.frame(data.table::rbindlist(rand))
+        randomxy <- data.frame(data.table::rbindlist(rand), stringsAsFactors = TRUE)
       }
       if(shuffle.type == 2){ #This potentially replaces each block in the dataset with one of the blocks from a distribution described by shuffleUnit. Each block has the potential to appear in the dataset up to numShuffUnit times.
         
@@ -292,40 +291,40 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
         if(length(blockAbsence) == length(randVec1)){ #if all blocks were empty, then there's no need to go any further
           randomxy <-NULL
         }else{ #if there was at least one populated block
-        
-        rand.blockTab <- data.frame(randVec1)
-        rand <-apply(rand.blockTab,1,xy.assignShuff, idFrame, dataType, numVertices)
-        randCoord<- data.frame(data.table::rbindlist(rand))
-        
-        blockFrame.full<-subset(y, block <= blocksPerUnit)
-        
-        if(length(blockAbsence > 0)){ #So, if there is a missing block
-          timestep.per.block <- length(unique(blockFrame.full$dateTime))/length(unique(blockFrame.full$block)) #pulls the number of timesteps per block. In the event that randVec pulls a starting block(s) that does not exist for the individual, NAs will be returned. Assuming that all timesteps are equidistant (as they should be when shuffleType == 2), this can only be the case if a block is pulled from a time before the individual began reporting tracks.
-          for(a in blockAbsence){ #Put NAs into randCoord where needed
-            randCoord.brkPt <-timestep.per.block*(a-1) #Identifies the row immediately prior to where NAs will be placed
-            naBlock<-data.frame(x.rand<-rep(NA, timestep.per.block), y.rand<-rep(NA, timestep.per.block)) #add NAs to columns when applicable
-            if(randCoord.brkPt == 0){ #if the first block is missing
-              randCoord<-data.frame(data.table::rbindlist(list(naBlock, randCoord)))
-            }else{ #if a block other than the 1st block is missing
-              if(a < length(randVec1)){ #if the missing block is not the last recorded one
-                preNACoord<- randCoord[1:randCoord.brkPt,]
-                postNACoord<- randCoord[(randCoord.brkPt+1):ncol(randCoord),]
-                randCoord<-data.frame(data.table::rbindlist(list(preNACoord, naBlock, postNACoord)))
+          
+          rand.blockTab <- data.frame(randVec1, stringsAsFactors = TRUE)
+          rand <-apply(rand.blockTab,1,xy.assignShuff, idFrame, dataType, numVertices)
+          randCoord<- data.frame(data.table::rbindlist(rand), stringsAsFactors = TRUE)
+          
+          blockFrame.full<-droplevels(subset(y, block <= blocksPerUnit)) #we need to pull the dateTimes for these blocks
+          
+          if(length(blockAbsence > 0)){ #So, if there is a missing block
+            timestep.per.block <- length(unique(blockFrame.full$dateTime))/length(unique(blockFrame.full$block)) #pulls the number of timesteps per block. In the event that randVec pulls a starting block(s) that does not exist for the individual, NAs will be returned. Assuming that all timesteps are equidistant (as they should be when shuffleType == 2), this can only be the case if a block is pulled from a time before the individual began reporting tracks.
+            for(a in blockAbsence){ #Put NAs into randCoord where needed
+              randCoord.brkPt <-timestep.per.block*(a-1) #Identifies the row immediately prior to where NAs will be placed
+              naBlock<-data.frame(x.rand<-rep(NA, timestep.per.block), y.rand<-rep(NA, timestep.per.block), stringsAsFactors = TRUE) #add NAs to columns when applicable
+              if(randCoord.brkPt == 0){ #if the first block is missing
+                randCoord<-data.frame(data.table::rbindlist(list(naBlock, randCoord)), stringsAsFactors = TRUE)
+              }else{ #if a block other than the 1st block is missing
+                if(a < length(randVec1)){ #if the missing block is not the last recorded one
+                  preNACoord<- randCoord[1:randCoord.brkPt,]
+                  postNACoord<- randCoord[(randCoord.brkPt+1):ncol(randCoord),]
+                  randCoord<-data.frame(data.table::rbindlist(list(preNACoord, naBlock, postNACoord)), stringsAsFactors = TRUE)
+                }
+                if(a == length(randVec1)){ #if the missing block IS the last recorded one
+                  randCoord<-data.frame(data.table::rbindlist(list(randCoord, naBlock)), stringsAsFactors = TRUE)
+                }              
               }
-              if(a == length(randVec1)){ #if the missing block IS the last recorded one
-                randCoord<-data.frame(data.table::rbindlist(list(randCoord, naBlock)))
-              }              
             }
           }
-        }
-        #outTable<- idFrame[1:nrow(randCoord),] #If the final block of the dataset has fewer observations than other blocks, and the randomized locations used this final block, this reduces the number of rows in idFrame to fit the number of rows in randCoord
-        outTable<-data.frame(id = unname(unlist(x[1])), dateTime = unique(blockFrame.full$dateTime), blockLength = unique(blockFrame.full$blockLength)) #create a dataframe with the number of rows corresponding to the number of times individuals were observed. This data frame will contain essentially the same info. as idFrame, but will ensure that blocks are consistent for all individuals.
-        outTable<- outTable[1:nrow(randCoord),] #If the final block of the dataset has fewer observations than other blocks, and the randomized locations used this final block, this reduces the number of rows in idFrame to fit the number of rows in randCoord
-        
-        #now we have to redefine block information for the single shuffleUnit (so that users can recall the block information they set)
-        bindlist <- list(outTable, randCoord)
-        randomxy <- do.call("cbind", bindlist)
-        randomxy<-randomxy[order(randomxy$id, randomxy$dateTime),]
+          #outTable<-data.frame(id = unname(unlist(x[1])), dateTime = unique(blockFrame.full$dateTime), blockLength = unique(blockFrame.full$blockLength), stringsAsFactors = TRUE) #create a dataframe with the number of rows corresponding to the number of times individuals were observed. This data frame will contain essentially the same info. as idFrame, but will ensure that blocks are consistent for all individuals.
+          outTable<-data.frame(id = rep(unname(unlist(x[1])), length(unique(blockFrame.full$dateTime))), dateTime = unique(blockFrame.full$dateTime), stringsAsFactors = TRUE) #create a dataframe with the number of rows corresponding to the number of times individuals were observed. This data frame will contain essentially the same info. as idFrame, but will ensure that blocks are consistent for all individuals.
+          outTable<- outTable[1:nrow(randCoord),] #If the final block of the dataset has fewer observations than other blocks, and the randomized locations used this final block, this reduces the number of rows in idFrame to fit the number of rows in randCoord
+          
+          #now we have to redefine block information for the single shuffleUnit (so that users can recall the block information they set)
+          bindlist <- list(outTable, randCoord)
+          randomxy <- do.call("cbind", bindlist)
+          randomxy<-randomxy[order(randomxy$id, randomxy$dateTime),]
         }
       }
     }else{ #if blocking == FALSE
@@ -333,18 +332,18 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
         rand.noblock <- sample(seq(1,nrow(idFrame),1), nrow(idFrame), replace = FALSE)
         rand.x <- idFrame$x[rand.noblock]
         rand.y <- idFrame$y[rand.noblock]
-        randomxy <- data.frame(x = rand.x, y = rand.y)
+        randomxy <- data.frame(x = rand.x, y = rand.y, stringsAsFactors = TRUE)
         colnames(randomxy) <- c("x.rand","y.rand")
       }
       if(dataType == "Polygon" || dataType == "POLYGON" || dataType == "polygon"){
         rand.noblock <- sample(seq(1,nrow(idFrame),1), nrow(idFrame), replace = FALSE)
-        rand.poly <- data.frame(matrix(ncol = 0, nrow = length(rand.noblock)))
+        rand.poly <- data.frame(matrix(ncol = 0, nrow = length(rand.noblock)), stringsAsFactors = TRUE)
         vertex <-1 #This is the base column number that, if added to "a" in the following loop will call the appropriate xy coordinates.
         for(a in 1:numVertices){
           rand.x <- idFrame[c(rand.noblock),(vertex+a)]
           rand.y <- idFrame[c(rand.noblock),(vertex+a+1)]
           vertex<- vertex+1
-          rand.point <-data.frame(rand.x, rand.y)
+          rand.point <-data.frame(rand.x, rand.y, stringsAsFactors = TRUE)
           colnames(rand.point)<- c(paste("point",a,".x", sep = ""), paste("point",a,".y", sep = ""))
           bindlist <-list(rand.poly, rand.point)
           rand.poly <- do.call("cbind", bindlist)
@@ -358,21 +357,21 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(dataType == "Point" || dataType == "POINT" || dataType == "point"){
       xVec <-y$x[which(y$block == unlist(unname(x[1])))]
       yVec <-y$y[which(y$block == unlist(unname(x[1])))]
-      xyrand <- data.frame(x.rand = xVec, y.rand = yVec)
+      xyrand <- data.frame(x.rand = xVec, y.rand = yVec, stringsAsFactors = TRUE)
     }
     if(dataType == "Polygon" || dataType == "POLYGON" || dataType == "polygon"){
       xTab <-y[which(y$block == x[1]),c(seq(2,(numVertices*2),2))]
       yTab <-y[which(y$block == x[1]),c(seq(3,(numVertices*2)+1,2))]
-      polygon <-data.frame(NULL)
-      for(i in 1:numVertices){
+      polygon <- data.frame(xTab[,1], yTab[,1], stringsAsFactors = TRUE) #start making the polygon by adding the first vertex
+      for(i in 2:numVertices){
         vert.x <-xTab[,i]
         vert.y <-yTab[,i]
         bindlist <-list(polygon,vert.x,vert.y)
         polygon<-do.call("cbind", bindlist)
       }
-      xyrand <- data.frame(polygon)
-      colnames(xyrand)[c(seq(1,(numVertices*2)-1,2))]<-paste("point",seq(1,numVertices,1),".x",sep ="")
-      colnames(xyrand)[c(seq(2,(numVertices*2),2))]<-paste("point",seq(1,numVertices,1),".y",sep ="")
+      xyrand <- data.frame(polygon, stringsAsFactors = TRUE)
+      colnames(xyrand)[c(seq(1,(numVertices*2)-1,2))]<-paste("point",seq(1,numVertices,1),".x_rand",sep ="")
+      colnames(xyrand)[c(seq(2,(numVertices*2),2))]<-paste("point",seq(1,numVertices,1),".y_rand",sep ="")
     }
     return(xyrand)
   }
@@ -383,38 +382,38 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(dataType == "Point" || dataType == "POINT" || dataType == "point"){
       xVec <-blockFrame$x[randSeq]
       yVec <-blockFrame$y[randSeq]
-      xyrand <- data.frame(x = xVec, y = yVec)
+      xyrand <- data.frame(x = xVec, y = yVec, stringsAsFactors = TRUE)
       colnames(xyrand) <- c("x.rand","y.rand")
     }
     if(dataType == "Polygon" || dataType == "POLYGON" || dataType == "polygon"){
       xTab <-blockFrame[randSeq,c(seq(2,(numVertices*2),2))]
       yTab <-blockFrame[randSeq,c(seq(3,(numVertices*2)+1,2))]
-      polygon <-data.frame(NULL)
-      for(i in 1:numVertices){
+      polygon <- data.frame(xTab[,1], yTab[,1], stringsAsFactors = TRUE) #start making the polygon by adding the first vertex
+      for(i in 2:numVertices){
         vert.x <-xTab[,i]
         vert.y <-yTab[,i]
         bindlist <-list(polygon,vert.x,vert.y)
         polygon<-do.call("cbind", bindlist)
       }
-      xyrand <- data.frame(polygon)
-      colnames(xyrand)[c(seq(1,(numVertices*2)-1,2))]<-paste("point",seq(1,numVertices,1),".x",sep ="")
-      colnames(xyrand)[c(seq(2,(numVertices*2),2))]<-paste("point",seq(1,numVertices,1),".y",sep ="")
+      xyrand <- data.frame(polygon, stringsAsFactors = TRUE)
+      colnames(xyrand)[c(seq(1,(numVertices*2)-1,2))]<-paste("point",seq(1,numVertices,1),".x_rand",sep ="")
+      colnames(xyrand)[c(seq(2,(numVertices*2),2))]<-paste("point",seq(1,numVertices,1),".y_rand",sep ="")
     }
     return(xyrand)
   }  		
   
   if(length(x) == 0){ #This if statement allows users to input either a series of vectors (id, dateTime, point.x and point.y), a dataframe with columns named the same, or a combination of dataframe and vectors. No matter the input format, a table called "originTab" will be created.
     if(dataType == "point" || dataType == "Point" || dataType == "POINT"){
-      originTab = data.frame(id = id, x = point.x, y = point.y, dateTime = dateTime)
+      originTab = data.frame(id = id, x = point.x, y = point.y, dateTime = dateTime, stringsAsFactors = TRUE)
     }
     if(dataType == "polygon" || dataType == "Polygon" || dataType == "POLYGON"){
-      originTab = data.frame(matrix(ncol = 0, nrow = length(id)))
+      originTab = data.frame(matrix(ncol = 0, nrow = length(id)), stringsAsFactors = TRUE)
       originTab$id = id
       colnames(poly.xy)[seq(1,(ncol(poly.xy) - 1),2)] = paste("point",seq(1,(ncol(poly.xy)/2),1),".x", sep = "")
       colnames(poly.xy)[seq(2,ncol(poly.xy),2)] = paste("point",seq(1,(ncol(poly.xy)/2),1),".y", sep = "")
-      dateFrame = data.frame(dateTime = dateTime)
+      dateFrame = data.frame(dateTime = dateTime, stringsAsFactors = TRUE)
       bindlist = list(originTab,poly.xy,dateFrame)
-      originTab = data.frame(do.call("cbind", bindlist))
+      originTab = data.frame(do.call("cbind", bindlist), stringsAsFactors = TRUE)
     }
   }
   
@@ -424,7 +423,7 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
         x$id <- x[,match(id, names(x))]
       }else{ #if length(id) > 1
         x$id = id
-      }
+      } 
     }
     idVec1 <- x$id
     
@@ -443,16 +442,16 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
           x$y = point.y
         }
       }
-      xyFrame1<- data.frame(x = x$x, y = x$y)
+      xyFrame1<- data.frame(x = x$x, y = x$y, stringsAsFactors = TRUE)
     }
     
     if(dataType == "polygon" || dataType == "Polygon" || dataType == "POLYGON"){
       
       if(length(poly.xy) > 0){
-        if(length(poly.xy) == (numVertices*2) & length(which(is.na(match(poly.xy, names(x)))) == TRUE) == 0){ 
+        if(length(as.matrix(poly.xy)) == (numVertices*2) && length(which(is.na(match(as.matrix(poly.xy), names(x)))) == TRUE) == 0){ 
           xyFrame1<-x[,match(poly.xy,names(x))] 
         }else{
-          xyFrame1<- data.frame(poly.xy)
+          xyFrame1<- data.frame(poly.xy, stringsAsFactors = TRUE)
         }
       }else{ #if length(poly.xy == 0)
         xyFrame1 <-  x[,(match("point1.x", names(x))):((2*numVertices) + (match("point1.x", names(x)) -1))] #if there is no poly.xy input, the code assumes the input is output from referencePointToPolygon function, and therefore, the first point of interest would be "point1.x"
@@ -491,10 +490,17 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(blockUnit == "Weeks" || blockUnit == "WEEKS" || blockUnit == "weeks"){
       blockLength1 <- blockLength*60*60*24*7 #num seconds in a week
     }
-    originTab<-originTab[order(originTab$dateTime),] #Just in case the data wasn't already ordered in this way.
-    originTab<-datetime.append1(originTab) #adds the total second column to the dataframe
-    studySecond <- (originTab$totalSecond -min(originTab$totalSecond)) + 1
-    originTab<-originTab[,-match("totalSecond", names(originTab))]
+    
+    daySecondList = lubridate::hour(originTab$dateTime) * 3600 + lubridate::minute(originTab$dateTime) * 60 + lubridate::second(originTab$dateTime) #This calculates a day-second
+    lub.dates = lubridate::date(originTab$dateTime)
+    originTab<-originTab[order(lub.dates, daySecondList),] #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data
+    rm(list = c("daySecondList", "lub.dates")) #remove these objects because they are no longer needed.
+    
+    #for some odd reason, difftime will output mostly zeroes (incorrectly) if there are > 1 correct 0 at the beginning. We use a crude fix here to address this. Basically, we create the zeroes first and combine it with other values afterwards
+    totSecond <- rep(0, length(which(originTab$dateTime == originTab$dateTime[1])))
+    totSecond2<-as.integer(difftime(originTab$dateTime[(length(totSecond) +1): nrow(originTab)] ,originTab$dateTime[1] , units = c("secs")))
+    studySecond <- as.integer((c(totSecond, totSecond2) -min(c(totSecond, totSecond2))) + 1)
+    
     numblocks <- ceiling((max(studySecond) - 1)/blockLength1)
     block <-rep(0,length(studySecond))
     for(g in 1:(numblocks -1)){ #numblocks - 1 because the last block in the dataset may be smaller than previous blocks (if blockLength1 does not divide evenly into timedif)
@@ -503,26 +509,20 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(length(which(block == 0)) > 0){ #identifies the last block
       block[which(block == 0)] = numblocks
     }
-    blockVec<-unique(block)
-    dateTimeVec2<-originTab$dateTime
-    minBlockTimeSeq <- rep(0, length(block)) #Added 2/4/2019. This vector will identify the minimum timepoint in each block.
-    maxBlockTimeSeq <- rep(0, length(block)) #Added 2/4/2019. This vector will identify the maximum timepoint in each block.
-    for(f in 1:length(blockVec)){
-      minBlockTime<-dateTimeVec2[min(which(block == blockVec[f]))]
-      minBlockTimeSeq[which(block == blockVec[f])] <- minBlockTime
-      maxBlockTime<-dateTimeVec2[max(which(block == blockVec[f]))]
-      maxBlockTimeSeq[which(block == blockVec[f])] <- maxBlockTime
-    }
+    
+    block.start<-as.character(as.POSIXct(originTab$dateTime[1]) + ((block - 1)*blockLength1)) #identify the timepoint where each block starts (down to the second resolution)
+    block.end<-as.character(as.POSIXct(originTab$dateTime[1]) + ((block - 1)*blockLength1) + (blockLength1 -1)) #identify the timepoint where each block ends (down to the second resolution)
+    
     originTab$block <- block
-    originTab$block.start <- minBlockTimeSeq
-    originTab$block.end <- maxBlockTimeSeq
-    originTab$numBlocks <- max(blockVec) 
-    originTab$blockLength <- paste(blockLength, blockUnit, sep = " ")
+    originTab$block.start <- block.start
+    originTab$block.end <- block.end
+    originTab$numBlocks <- max(block) #the contactTest function will require this information (i.e. the number of blocks in the dataset)
+    
   }
   originTab <- originTab[order(originTab$id, originTab$dateTime),] #order by id, then dateTime
   
   if(numRandomizations > 1){
-    repeatTab<- data.frame(seq(1,numRandomizations, 1))
+    repeatTab<- data.frame(seq(1,numRandomizations, 1), stringsAsFactors = TRUE)
     repeatTab$dataType = dataType
     repeatTab$numVertices = numVertices
     repeatTab$shuffle.type = shuffle.type
@@ -531,11 +531,12 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     repeatTab$subUnit = shuffleUnit
     repeatTab$blockUnit = blockUnit
     repeatTab$blockLength = blockLength
+    repeatTab$reduceOutput = reduceOutput
     
     if(parallel == TRUE){
       cl<-parallel::makeCluster(nCores)
+      on.exit(parallel::stopCluster(cl))
       randomness<-parallel::parApply(cl, repeatTab, 1, randomization, originTab)
-      parallel::stopCluster(cl)
     }else{
       randomness <- apply(repeatTab, 1, randomization, originTab)
     }
@@ -548,12 +549,12 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
       idVecSeq <- 1
     }
     numids <- length(idVecSeq)
-    idTab<- data.frame(idVecSeq)
+    idTab<- data.frame(idVecSeq, stringsAsFactors = TRUE)
     
     if(parallel == TRUE){
       cl<-parallel::makeCluster(nCores)
+      on.exit(parallel::stopCluster(cl))
       randomness<-parallel::parApply(cl, idTab,1,datasub.func, originTab, numids,blocking, shuffle.type, dataType, shuffleUnit, blockUnit, blockLength)
-      parallel::stopCluster(cl)
     }else{
       randomness <- apply(idTab,1,datasub.func, originTab, numids,blocking, shuffle.type, dataType, shuffleUnit, blockUnit, blockLength)
     }
@@ -561,7 +562,7 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
     if(is.data.frame(randomness) == TRUE){
       randomxyCoords<-randomness
     }else{
-      randomxyCoords<-data.frame(data.table::rbindlist(randomness))
+      randomxyCoords<-data.frame(data.table::rbindlist(randomness), stringsAsFactors = TRUE)
     }
     
     if(blocking == TRUE & shuffle.type == 2){ #To account for potential length differences in blocks if shuffle.type ==2, the outputTab is compiled earlier on in the function (in the datasub.func subfunction) 
@@ -576,6 +577,13 @@ randomizePaths<-function(x = NULL, id = NULL, dateTime = NULL, point.x = NULL, p
       }
     }
     outputTab$rand.rep = 1    
+    
+    if(reduceOutput == TRUE){
+      outputTab <- data.frame(id = outputTab$id, x.rand = outputTab$x.rand, y.rand = outputTab$y.rand, dateTime = outputTab$dateTime, rand.rep = outputTab$rand.rep, stringsAsFactors = TRUE)
+    }else{
+      rownames(outputTab)<- seq(1, nrow(outputTab), 1) #ensure rownames are sequential. If output was reduced, this happened automatically.
+    }
+    
     return(outputTab)
   }
 }
